@@ -8,14 +8,14 @@ from django.contrib.auth.tokens import PasswordResetTokenGenerator
 from django.contrib.sites.shortcuts import get_current_site
 from django.core import exceptions, validators
 from django.urls import reverse
-from django.utils.translation import pgettext, ugettext, ugettext_lazy as _
+from django.utils.translation import gettext, gettext_lazy as _, pgettext
 
-from . import app_settings
 from ..utils import (
     build_absolute_uri,
     get_username_max_length,
     set_form_field_order,
 )
+from . import app_settings
 from .adapter import get_adapter
 from .app_settings import AuthenticationMethod
 from .models import EmailAddress
@@ -33,18 +33,17 @@ from .utils import (
 
 
 class EmailAwarePasswordResetTokenGenerator(PasswordResetTokenGenerator):
-
     def _make_hash_value(self, user, timestamp):
-        ret = super(
-            EmailAwarePasswordResetTokenGenerator, self)._make_hash_value(
-                user, timestamp)
+        ret = super(EmailAwarePasswordResetTokenGenerator, self)._make_hash_value(
+            user, timestamp
+        )
         sync_user_email_addresses(user)
-        emails = set([user.email])
+        email = user_email(user)
+        emails = set([email] if email else [])
         emails.update(
-            EmailAddress.objects
-            .filter(user=user)
-            .values_list('email', flat=True))
-        ret += '|'.join(sorted(emails))
+            EmailAddress.objects.filter(user=user).values_list("email", flat=True)
+        )
+        ret += "|".join(sorted(emails))
         return ret
 
 
@@ -54,29 +53,31 @@ default_token_generator = EmailAwarePasswordResetTokenGenerator()
 class PasswordVerificationMixin(object):
     def clean(self):
         cleaned_data = super(PasswordVerificationMixin, self).clean()
-        password1 = cleaned_data.get('password1')
-        password2 = cleaned_data.get('password2')
+        password1 = cleaned_data.get("password1")
+        password2 = cleaned_data.get("password2")
         if (password1 and password2) and password1 != password2:
-            self.add_error(
-                'password2', _("You must type the same password each time.")
-            )
+            self.add_error("password2", _("You must type the same password each time."))
         return cleaned_data
 
 
 class PasswordField(forms.CharField):
-
     def __init__(self, *args, **kwargs):
-        render_value = kwargs.pop('render_value',
-                                  app_settings.PASSWORD_INPUT_RENDER_VALUE)
-        kwargs['widget'] = forms.PasswordInput(render_value=render_value,
-                                               attrs={'placeholder':
-                                                      kwargs.get("label")})
+        render_value = kwargs.pop(
+            "render_value", app_settings.PASSWORD_INPUT_RENDER_VALUE
+        )
+        kwargs["widget"] = forms.PasswordInput(
+            render_value=render_value,
+            attrs={"placeholder": kwargs.get("label")},
+        )
+        autocomplete = kwargs.pop("autocomplete", None)
+        if autocomplete is not None:
+            kwargs["widget"].attrs["autocomplete"] = autocomplete
         super(PasswordField, self).__init__(*args, **kwargs)
 
 
 class SetPasswordField(PasswordField):
-
     def __init__(self, *args, **kwargs):
+        kwargs["autocomplete"] = "new-password"
         super(SetPasswordField, self).__init__(*args, **kwargs)
         self.user = None
 
@@ -88,54 +89,56 @@ class SetPasswordField(PasswordField):
 
 class LoginForm(forms.Form):
 
-    password = PasswordField(label=_("Password"))
-    remember = forms.BooleanField(label=_("Remember Me"),
-                                  required=False)
+    password = PasswordField(label=_("Password"), autocomplete="current-password")
+    remember = forms.BooleanField(label=_("Remember Me"), required=False)
 
     user = None
     error_messages = {
-        'account_inactive':
-        _("This account is currently inactive."),
-
-        'email_password_mismatch':
-        _("The e-mail address and/or password you specified are not correct."),
-
-        'username_password_mismatch':
-        _("The username and/or password you specified are not correct."),
+        "account_inactive": _("This account is currently inactive."),
+        "email_password_mismatch": _(
+            "The e-mail address and/or password you specified are not correct."
+        ),
+        "username_password_mismatch": _(
+            "The username and/or password you specified are not correct."
+        ),
     }
 
     def __init__(self, *args, **kwargs):
-        self.request = kwargs.pop('request', None)
+        self.request = kwargs.pop("request", None)
         super(LoginForm, self).__init__(*args, **kwargs)
         if app_settings.AUTHENTICATION_METHOD == AuthenticationMethod.EMAIL:
-            login_widget = forms.TextInput(attrs={'type': 'email',
-                                                  'placeholder':
-                                                  _('E-mail address'),
-                                                  'autofocus': 'autofocus'})
-            login_field = forms.EmailField(label=_("E-mail"),
-                                           widget=login_widget)
-        elif app_settings.AUTHENTICATION_METHOD \
-                == AuthenticationMethod.USERNAME:
-            login_widget = forms.TextInput(attrs={'placeholder':
-                                                  _('Username'),
-                                                  'autofocus': 'autofocus'})
+            login_widget = forms.TextInput(
+                attrs={
+                    "type": "email",
+                    "placeholder": _("E-mail address"),
+                    "autocomplete": "email",
+                }
+            )
+            login_field = forms.EmailField(label=_("E-mail"), widget=login_widget)
+        elif app_settings.AUTHENTICATION_METHOD == AuthenticationMethod.USERNAME:
+            login_widget = forms.TextInput(
+                attrs={"placeholder": _("Username"), "autocomplete": "username"}
+            )
             login_field = forms.CharField(
                 label=_("Username"),
                 widget=login_widget,
-                max_length=get_username_max_length())
+                max_length=get_username_max_length(),
+            )
         else:
-            assert app_settings.AUTHENTICATION_METHOD \
+            assert (
+                app_settings.AUTHENTICATION_METHOD
                 == AuthenticationMethod.USERNAME_EMAIL
-            login_widget = forms.TextInput(attrs={'placeholder':
-                                                  _('Username or e-mail'),
-                                                  'autofocus': 'autofocus'})
-            login_field = forms.CharField(label=pgettext("field label",
-                                                         "Login"),
-                                          widget=login_widget)
+            )
+            login_widget = forms.TextInput(
+                attrs={"placeholder": _("Username or e-mail"), "autocomplete": "email"}
+            )
+            login_field = forms.CharField(
+                label=pgettext("field label", "Login"), widget=login_widget
+            )
         self.fields["login"] = login_field
         set_form_field_order(self, ["login", "password", "remember"])
         if app_settings.SESSION_REMEMBER is not None:
-            del self.fields['remember']
+            del self.fields["remember"]
 
     def user_credentials(self):
         """
@@ -146,9 +149,7 @@ class LoginForm(forms.Form):
         login = self.cleaned_data["login"]
         if app_settings.AUTHENTICATION_METHOD == AuthenticationMethod.EMAIL:
             credentials["email"] = login
-        elif (
-                app_settings.AUTHENTICATION_METHOD ==
-                AuthenticationMethod.USERNAME):
+        elif app_settings.AUTHENTICATION_METHOD == AuthenticationMethod.USERNAME:
             credentials["username"] = login
         else:
             if self._is_login_email(login):
@@ -158,7 +159,7 @@ class LoginForm(forms.Form):
         return credentials
 
     def clean_login(self):
-        login = self.cleaned_data['login']
+        login = self.cleaned_data["login"]
         return login.strip()
 
     def _is_login_email(self, login):
@@ -174,30 +175,34 @@ class LoginForm(forms.Form):
         if self._errors:
             return
         credentials = self.user_credentials()
-        user = get_adapter(self.request).authenticate(
-            self.request,
-            **credentials)
+        user = get_adapter(self.request).authenticate(self.request, **credentials)
         if user:
             self.user = user
         else:
             auth_method = app_settings.AUTHENTICATION_METHOD
             if auth_method == app_settings.AuthenticationMethod.USERNAME_EMAIL:
-                login = self.cleaned_data['login']
+                login = self.cleaned_data["login"]
                 if self._is_login_email(login):
                     auth_method = app_settings.AuthenticationMethod.EMAIL
                 else:
                     auth_method = app_settings.AuthenticationMethod.USERNAME
             raise forms.ValidationError(
-                self.error_messages['%s_password_mismatch' % auth_method])
+                self.error_messages["%s_password_mismatch" % auth_method]
+            )
         return self.cleaned_data
 
     def login(self, request, redirect_url=None):
-        ret = perform_login(request, self.user,
-                            email_verification=app_settings.EMAIL_VERIFICATION,
-                            redirect_url=redirect_url)
+        email = self.user_credentials().get("email")
+        ret = perform_login(
+            request,
+            self.user,
+            email_verification=app_settings.EMAIL_VERIFICATION,
+            redirect_url=redirect_url,
+            email=email,
+        )
         remember = app_settings.SESSION_REMEMBER
         if remember is None:
-            remember = self.cleaned_data['remember']
+            remember = self.cleaned_data["remember"]
         if remember:
             request.session.set_expiry(app_settings.SESSION_COOKIE_AGE)
         else:
@@ -206,7 +211,6 @@ class LoginForm(forms.Form):
 
 
 class _DummyCustomSignupForm(forms.Form):
-
     def signup(self, request, user):
         """
         Invoked at signup time to complete the signup of the user.
@@ -229,96 +233,107 @@ def _base_signup_form_class():
     if not app_settings.SIGNUP_FORM_CLASS:
         return _DummyCustomSignupForm
     try:
-        fc_module, fc_classname = app_settings.SIGNUP_FORM_CLASS.rsplit('.', 1)
+        fc_module, fc_classname = app_settings.SIGNUP_FORM_CLASS.rsplit(".", 1)
     except ValueError:
-        raise exceptions.ImproperlyConfigured('%s does not point to a form'
-                                              ' class'
-                                              % app_settings.SIGNUP_FORM_CLASS)
+        raise exceptions.ImproperlyConfigured(
+            "%s does not point to a form class" % app_settings.SIGNUP_FORM_CLASS
+        )
     try:
         mod = import_module(fc_module)
     except ImportError as e:
-        raise exceptions.ImproperlyConfigured('Error importing form class %s:'
-                                              ' "%s"' % (fc_module, e))
+        raise exceptions.ImproperlyConfigured(
+            "Error importing form class %s:" ' "%s"' % (fc_module, e)
+        )
     try:
         fc_class = getattr(mod, fc_classname)
     except AttributeError:
-        raise exceptions.ImproperlyConfigured('Module "%s" does not define a'
-                                              ' "%s" class' % (fc_module,
-                                                               fc_classname))
-    if not hasattr(fc_class, 'signup'):
-        if hasattr(fc_class, 'save'):
-            warnings.warn("The custom signup form must offer"
-                          " a `def signup(self, request, user)` method",
-                          DeprecationWarning)
+        raise exceptions.ImproperlyConfigured(
+            'Module "%s" does not define a' ' "%s" class' % (fc_module, fc_classname)
+        )
+    if not hasattr(fc_class, "signup"):
+        if hasattr(fc_class, "save"):
+            warnings.warn(
+                "The custom signup form must offer"
+                " a `def signup(self, request, user)` method",
+                DeprecationWarning,
+            )
         else:
             raise exceptions.ImproperlyConfigured(
-                'The custom signup form must implement a "signup" method')
+                'The custom signup form must implement a "signup" method'
+            )
     return fc_class
 
 
 class BaseSignupForm(_base_signup_form_class()):
-    username = forms.CharField(label=_("Username"),
-                               min_length=app_settings.USERNAME_MIN_LENGTH,
-                               widget=forms.TextInput(
-                                   attrs={'placeholder':
-                                          _('Username'),
-                                          'autofocus': 'autofocus'}))
-    email = forms.EmailField(widget=forms.TextInput(
-        attrs={'type': 'email',
-               'placeholder': _('E-mail address')}))
+    username = forms.CharField(
+        label=_("Username"),
+        min_length=app_settings.USERNAME_MIN_LENGTH,
+        widget=forms.TextInput(
+            attrs={"placeholder": _("Username"), "autocomplete": "username"}
+        ),
+    )
+    email = forms.EmailField(
+        widget=forms.TextInput(
+            attrs={
+                "type": "email",
+                "placeholder": _("E-mail address"),
+                "autocomplete": "email",
+            }
+        )
+    )
 
     def __init__(self, *args, **kwargs):
-        email_required = kwargs.pop('email_required',
-                                    app_settings.EMAIL_REQUIRED)
-        self.username_required = kwargs.pop('username_required',
-                                            app_settings.USERNAME_REQUIRED)
+        email_required = kwargs.pop("email_required", app_settings.EMAIL_REQUIRED)
+        self.username_required = kwargs.pop(
+            "username_required", app_settings.USERNAME_REQUIRED
+        )
         super(BaseSignupForm, self).__init__(*args, **kwargs)
-        username_field = self.fields['username']
+        username_field = self.fields["username"]
         username_field.max_length = get_username_max_length()
         username_field.validators.append(
-            validators.MaxLengthValidator(username_field.max_length))
-        username_field.widget.attrs['maxlength'] = str(
-            username_field.max_length)
+            validators.MaxLengthValidator(username_field.max_length)
+        )
+        username_field.widget.attrs["maxlength"] = str(username_field.max_length)
 
         default_field_order = [
-            'email',
-            'email2',  # ignored when not present
-            'username',
-            'password1',
-            'password2'  # ignored when not present
+            "email",
+            "email2",  # ignored when not present
+            "username",
+            "password1",
+            "password2",  # ignored when not present
         ]
         if app_settings.SIGNUP_EMAIL_ENTER_TWICE:
             self.fields["email2"] = forms.EmailField(
                 label=_("E-mail (again)"),
                 widget=forms.TextInput(
                     attrs={
-                        'type': 'email',
-                        'placeholder': _('E-mail address confirmation')
+                        "type": "email",
+                        "placeholder": _("E-mail address confirmation"),
                     }
-                )
+                ),
             )
         if email_required:
-            self.fields['email'].label = ugettext("E-mail")
-            self.fields['email'].required = True
+            self.fields["email"].label = gettext("E-mail")
+            self.fields["email"].required = True
         else:
-            self.fields['email'].label = ugettext("E-mail (optional)")
-            self.fields['email'].required = False
-            self.fields['email'].widget.is_required = False
+            self.fields["email"].label = gettext("E-mail (optional)")
+            self.fields["email"].required = False
+            self.fields["email"].widget.is_required = False
             if self.username_required:
                 default_field_order = [
-                    'username',
-                    'email',
-                    'email2',  # ignored when not present
-                    'password1',
-                    'password2'  # ignored when not present
+                    "username",
+                    "email",
+                    "email2",  # ignored when not present
+                    "password1",
+                    "password2",  # ignored when not present
                 ]
 
         if not self.username_required:
             del self.fields["username"]
 
         set_form_field_order(
-            self,
-            getattr(self, 'field_order', None) or default_field_order)
+            self, getattr(self, "field_order", None) or default_field_order
+        )
 
     def clean_username(self):
         value = self.cleaned_data["username"]
@@ -326,7 +341,7 @@ class BaseSignupForm(_base_signup_form_class()):
         return value
 
     def clean_email(self):
-        value = self.cleaned_data['email']
+        value = self.cleaned_data["email"]
         value = get_adapter().clean_email(value)
         if value and app_settings.UNIQUE_EMAIL:
             value = self.validate_unique_email(value)
@@ -338,22 +353,22 @@ class BaseSignupForm(_base_signup_form_class()):
     def clean(self):
         cleaned_data = super(BaseSignupForm, self).clean()
         if app_settings.SIGNUP_EMAIL_ENTER_TWICE:
-            email = cleaned_data.get('email')
-            email2 = cleaned_data.get('email2')
+            email = cleaned_data.get("email")
+            email2 = cleaned_data.get("email2")
             if (email and email2) and email != email2:
-                self.add_error(
-                    'email2', _("You must type the same email each time.")
-                )
+                self.add_error("email2", _("You must type the same email each time."))
         return cleaned_data
 
     def custom_signup(self, request, user):
         custom_form = super(BaseSignupForm, self)
-        if hasattr(custom_form, 'signup') and callable(custom_form.signup):
+        if hasattr(custom_form, "signup") and callable(custom_form.signup):
             custom_form.signup(request, user)
         else:
-            warnings.warn("The custom signup form must offer"
-                          " a `def signup(self, request, user)` method",
-                          DeprecationWarning)
+            warnings.warn(
+                "The custom signup form must offer"
+                " a `def signup(self, request, user)` method",
+                DeprecationWarning,
+            )
             # Historically, it was called .save, but this is confusing
             # in case of ModelForm
             custom_form.save(user)
@@ -362,12 +377,13 @@ class BaseSignupForm(_base_signup_form_class()):
 class SignupForm(BaseSignupForm):
     def __init__(self, *args, **kwargs):
         super(SignupForm, self).__init__(*args, **kwargs)
-        self.fields['password1'] = PasswordField(label=_("Password"))
+        self.fields["password1"] = PasswordField(
+            label=_("Password"), autocomplete="new-password"
+        )
         if app_settings.SIGNUP_PASSWORD_ENTER_TWICE:
-            self.fields['password2'] = PasswordField(
-                label=_("Password (again)"))
+            self.fields["password2"] = PasswordField(label=_("Password (again)"))
 
-        if hasattr(self, 'field_order'):
+        if hasattr(self, "field_order"):
             set_form_field_order(self, self.field_order)
 
     def clean(self):
@@ -379,23 +395,23 @@ class SignupForm(BaseSignupForm):
         dummy_user = get_user_model()
         user_username(dummy_user, self.cleaned_data.get("username"))
         user_email(dummy_user, self.cleaned_data.get("email"))
-        password = self.cleaned_data.get('password1')
+        password = self.cleaned_data.get("password1")
         if password:
             try:
-                get_adapter().clean_password(
-                    password,
-                    user=dummy_user)
+                get_adapter().clean_password(password, user=dummy_user)
             except forms.ValidationError as e:
-                self.add_error('password1', e)
+                self.add_error("password1", e)
 
-        if app_settings.SIGNUP_PASSWORD_ENTER_TWICE \
-                and "password1" in self.cleaned_data \
-                and "password2" in self.cleaned_data:
-            if self.cleaned_data["password1"] \
-                    != self.cleaned_data["password2"]:
+        if (
+            app_settings.SIGNUP_PASSWORD_ENTER_TWICE
+            and "password1" in self.cleaned_data
+            and "password2" in self.cleaned_data
+        ):
+            if self.cleaned_data["password1"] != self.cleaned_data["password2"]:
                 self.add_error(
-                    'password2',
-                    _("You must type the same password each time."))
+                    "password2",
+                    _("You must type the same password each time."),
+                )
         return self.cleaned_data
 
     def save(self, request):
@@ -409,7 +425,6 @@ class SignupForm(BaseSignupForm):
 
 
 class UserForm(forms.Form):
-
     def __init__(self, user=None, *args, **kwargs):
         self.user = user
         super(UserForm, self).__init__(*args, **kwargs)
@@ -421,18 +436,21 @@ class AddEmailForm(UserForm):
         label=_("E-mail"),
         required=True,
         widget=forms.TextInput(
-            attrs={"type": "email",
-                   "size": "30",
-                   "placeholder": _('E-mail address')}))
+            attrs={"type": "email", "placeholder": _("E-mail address")}
+        ),
+    )
 
     def clean_email(self):
         value = self.cleaned_data["email"]
         value = get_adapter().clean_email(value)
         errors = {
-            "this_account": _("This e-mail address is already associated"
-                              " with this account."),
-            "different_account": _("This e-mail address is already associated"
-                                   " with another account."),
+            "this_account": _(
+                "This e-mail address is already associated with this account."
+            ),
+            "different_account": _(
+                "This e-mail address is already associated with another account."
+            ),
+            "max_email_addresses": _("You cannot add more than %d e-mail addresses."),
         }
         users = filter_users_by_email(value)
         on_this_account = [u for u in users if u.pk == self.user.pk]
@@ -442,29 +460,33 @@ class AddEmailForm(UserForm):
             raise forms.ValidationError(errors["this_account"])
         if on_diff_account and app_settings.UNIQUE_EMAIL:
             raise forms.ValidationError(errors["different_account"])
+        if not EmailAddress.objects.can_add_email(self.user):
+            raise forms.ValidationError(
+                errors["max_email_addresses"] % app_settings.MAX_EMAIL_ADDRESSES
+            )
         return value
 
     def save(self, request):
-        return EmailAddress.objects.add_email(request,
-                                              self.user,
-                                              self.cleaned_data["email"],
-                                              confirm=True)
+        return EmailAddress.objects.add_email(
+            request, self.user, self.cleaned_data["email"], confirm=True
+        )
 
 
 class ChangePasswordForm(PasswordVerificationMixin, UserForm):
 
-    oldpassword = PasswordField(label=_("Current Password"))
+    oldpassword = PasswordField(
+        label=_("Current Password"), autocomplete="current-password"
+    )
     password1 = SetPasswordField(label=_("New Password"))
     password2 = PasswordField(label=_("New Password (again)"))
 
     def __init__(self, *args, **kwargs):
         super(ChangePasswordForm, self).__init__(*args, **kwargs)
-        self.fields['password1'].user = self.user
+        self.fields["password1"].user = self.user
 
     def clean_oldpassword(self):
         if not self.user.check_password(self.cleaned_data.get("oldpassword")):
-            raise forms.ValidationError(_("Please type your current"
-                                          " password."))
+            raise forms.ValidationError(_("Please type your current password."))
         return self.cleaned_data["oldpassword"]
 
     def save(self):
@@ -478,7 +500,7 @@ class SetPasswordForm(PasswordVerificationMixin, UserForm):
 
     def __init__(self, *args, **kwargs):
         super(SetPasswordForm, self).__init__(*args, **kwargs)
-        self.fields['password1'].user = self.user
+        self.fields["password1"].user = self.user
 
     def save(self):
         get_adapter().set_password(self.user, self.cleaned_data["password1"])
@@ -489,27 +511,29 @@ class ResetPasswordForm(forms.Form):
     email = forms.EmailField(
         label=_("E-mail"),
         required=True,
-        widget=forms.TextInput(attrs={
-            "type": "email",
-            "size": "30",
-            "placeholder": _("E-mail address"),
-        })
+        widget=forms.TextInput(
+            attrs={
+                "type": "email",
+                "placeholder": _("E-mail address"),
+                "autocomplete": "email",
+            }
+        ),
     )
 
     def clean_email(self):
         email = self.cleaned_data["email"]
         email = get_adapter().clean_email(email)
-        self.users = filter_users_by_email(email)
+        self.users = filter_users_by_email(email, is_active=True)
         if not self.users:
-            raise forms.ValidationError(_("The e-mail address is not assigned"
-                                          " to any user account"))
+            raise forms.ValidationError(
+                _("The e-mail address is not assigned to any user account")
+            )
         return self.cleaned_data["email"]
 
     def save(self, request, **kwargs):
         current_site = get_current_site(request)
         email = self.cleaned_data["email"]
-        token_generator = kwargs.get("token_generator",
-                                     default_token_generator)
+        token_generator = kwargs.get("token_generator", default_token_generator)
 
         for user in self.users:
 
@@ -520,24 +544,24 @@ class ResetPasswordForm(forms.Form):
             # password_reset.save()
 
             # send the password reset email
-            path = reverse("account_reset_password_from_key",
-                           kwargs=dict(uidb36=user_pk_to_url_str(user),
-                                       key=temp_key))
-            url = build_absolute_uri(
-                request, path)
+            path = reverse(
+                "account_reset_password_from_key",
+                kwargs=dict(uidb36=user_pk_to_url_str(user), key=temp_key),
+            )
+            url = build_absolute_uri(request, path)
 
-            context = {"current_site": current_site,
-                       "user": user,
-                       "password_reset_url": url,
-                       "request": request}
+            context = {
+                "current_site": current_site,
+                "user": user,
+                "password_reset_url": url,
+                "request": request,
+            }
 
-            if app_settings.AUTHENTICATION_METHOD \
-                    != AuthenticationMethod.EMAIL:
-                context['username'] = user_username(user)
+            if app_settings.AUTHENTICATION_METHOD != AuthenticationMethod.EMAIL:
+                context["username"] = user_username(user)
             get_adapter(request).send_mail(
-                'account/email/password_reset_key',
-                email,
-                context)
+                "account/email/password_reset_key", email, context
+            )
         return self.cleaned_data["email"]
 
 
@@ -550,7 +574,7 @@ class ResetPasswordKeyForm(PasswordVerificationMixin, forms.Form):
         self.user = kwargs.pop("user", None)
         self.temp_key = kwargs.pop("temp_key", None)
         super(ResetPasswordKeyForm, self).__init__(*args, **kwargs)
-        self.fields['password1'].user = self.user
+        self.fields["password1"].user = self.user
 
     def save(self):
         get_adapter().set_password(self.user, self.cleaned_data["password1"])
@@ -565,7 +589,7 @@ class UserTokenForm(forms.Form):
     token_generator = default_token_generator
 
     error_messages = {
-        'token_invalid': _('The password reset token was invalid.'),
+        "token_invalid": _("The password reset token was invalid."),
     }
 
     def _get_user(self, uidb36):
@@ -579,15 +603,16 @@ class UserTokenForm(forms.Form):
     def clean(self):
         cleaned_data = super(UserTokenForm, self).clean()
 
-        uidb36 = cleaned_data.get('uidb36', None)
-        key = cleaned_data.get('key', None)
+        uidb36 = cleaned_data.get("uidb36", None)
+        key = cleaned_data.get("key", None)
 
         if not key:
-            raise forms.ValidationError(self.error_messages['token_invalid'])
+            raise forms.ValidationError(self.error_messages["token_invalid"])
 
         self.reset_user = self._get_user(uidb36)
-        if (self.reset_user is None or
-                not self.token_generator.check_token(self.reset_user, key)):
-            raise forms.ValidationError(self.error_messages['token_invalid'])
+        if self.reset_user is None or not self.token_generator.check_token(
+            self.reset_user, key
+        ):
+            raise forms.ValidationError(self.error_messages["token_invalid"])
 
         return cleaned_data
