@@ -1,4 +1,4 @@
-from django.conf.urls import url
+from django.urls import path
 
 from allauth.socialaccount.providers.oauth2.urls import default_urlpatterns
 
@@ -9,7 +9,9 @@ from .provider import FacebookProvider
 urlpatterns = default_urlpatterns(FacebookProvider)
 
 urlpatterns += [
-    url(r'^facebook/login/token/$',
+    path(
+        "facebook/login/token/",
         views.login_by_token,
-        name="facebook_login_by_token"),
+        name="facebook_login_by_token",
+    ),
 ]
