@@ -5,25 +5,25 @@ from allauth.socialaccount.providers.oauth2.provider import OAuth2Provider
 
 
 class Scope(object):
-    EMAIL = 'email'
-    PROFILE = 'profile'
+    EMAIL = "email"
+    PROFILE = "profile"
 
 
 class GoogleAccount(ProviderAccount):
     def get_profile_url(self):
-        return self.account.extra_data.get('link')
+        return self.account.extra_data.get("link")
 
     def get_avatar_url(self):
-        return self.account.extra_data.get('picture')
+        return self.account.extra_data.get("picture")
 
     def to_str(self):
         dflt = super(GoogleAccount, self).to_str()
-        return self.account.extra_data.get('name', dflt)
+        return self.account.extra_data.get("name", dflt)
 
 
 class GoogleProvider(OAuth2Provider):
-    id = 'google'
-    name = 'Google'
+    id = "google"
+    name = "Google"
     account_class = GoogleAccount
 
     def get_default_scope(self):
@@ -33,27 +33,26 @@ class GoogleProvider(OAuth2Provider):
         return scope
 
     def get_auth_params(self, request, action):
-        ret = super(GoogleProvider, self).get_auth_params(request,
-                                                          action)
+        ret = super(GoogleProvider, self).get_auth_params(request, action)
         if action == AuthAction.REAUTHENTICATE:
-            ret['prompt'] = 'select_account'
+            ret["prompt"] = "select_account consent"
         return ret
 
     def extract_uid(self, data):
-        return str(data['id'])
+        return str(data["id"])
 
     def extract_common_fields(self, data):
-        return dict(email=data.get('email'),
-                    last_name=data.get('family_name'),
-                    first_name=data.get('given_name'))
+        return dict(
+            email=data.get("email"),
+            last_name=data.get("family_name"),
+            first_name=data.get("given_name"),
+        )
 
     def extract_email_addresses(self, data):
         ret = []
-        email = data.get('email')
-        if email and data.get('verified_email'):
-            ret.append(EmailAddress(email=email,
-                       verified=True,
-                       primary=True))
+        email = data.get("email")
+        if email and data.get("verified_email"):
+            ret.append(EmailAddress(email=email, verified=True, primary=True))
         return ret
 
 
