@@ -7,21 +7,21 @@ class TwentyThreeAndMeAccount(ProviderAccount):
 
 
 class TwentyThreeAndMeProvider(OAuth2Provider):
-    id = 'twentythreeandme'
-    slug = '23andme'
-    name = '23andMe'
+    id = "twentythreeandme"
+    slug = "23andme"
+    name = "23andMe"
     account_class = TwentyThreeAndMeAccount
 
     def extract_uid(self, data):
-        return data['id']
+        return data["id"]
 
     def get_default_scope(self):
-        scope = ['basic']
+        scope = ["basic"]
         return scope
 
     def extract_common_fields(self, data):
         return dict(
-            email=data.get('email'),
+            email=data.get("email"),
         )
 
 
