@@ -1,4 +1,3 @@
-from allauth.socialaccount import providers
 from allauth.socialaccount.providers.base import ProviderAccount
 from allauth.socialaccount.providers.oauth.provider import OAuthProvider
 
@@ -8,9 +7,8 @@ class VimeoAccount(ProviderAccount):
 
 
 class VimeoProvider(OAuthProvider):
-    id = 'vimeo'
-    name = 'Vimeo'
-    package = 'allauth.socialaccount.providers.vimeo'
+    id = "vimeo"
+    name = "Vimeo"
     account_class = VimeoAccount
 
     def get_default_scope(self):
@@ -18,11 +16,10 @@ class VimeoProvider(OAuthProvider):
         return scope
 
     def extract_uid(self, data):
-        return data['id']
+        return data["id"]
 
     def extract_common_fields(self, data):
-        return dict(name=data.get('display_name'),
-                    username=data.get('username'))
+        return dict(name=data.get("display_name"), username=data.get("username"))
 
 
-providers.registry.register(VimeoProvider)
+provider_classes = [VimeoProvider]
